#!/usr/bin/perl

# Please don't encrypt me!
  
use Filter::CBC "","";

# This file will be encrypted and overwritten.
# Make backups, damnit!
# Autofilter example
# Defaults will be used
# Rijndael is default encryption algorithm
# Default keyphrase is : This space is left blank intentionally
print "Don't try this at home, kids !";